==> wm-react-native-components-0.0.62.tgz: package/WMRadioGroup.js <==
/**
 * Created with IntelliJ IDEA.
 * User: aaronlin
 * Date: 2017/11/22
 * Time: 13:37
 */
import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';

import {
    View,
    Text,
    Image,
    TouchableWithoutFeedback,
    ViewPropTypes,
} from 'react-native';

import WMText from './WMText';

import {
    StyleSheetPropType,
    ViewStylePropTypes,
    TextStylePropTypes,
    ImageStylePropTypes,
} from './RNExt';

const viewStylePropType = StyleSheetPropType(ViewStylePropTypes);
const textStylePropType = StyleSheetPropType(TextStylePropTypes);
const imageStylePropTypes = StyleSheetPropType(ImageStylePropTypes);

/**
 * 单选Radio
 */
export default class WMRadioGroup extends PureComponent {

    static propTypes = {
        ...ViewPropTypes,
        data:PropTypes.arrayOf(PropTypes.shape({    //数据
            name:PropTypes.string.isRequired, //用来显示的name
        })),
        radioItemContainerStyle:viewStylePropType,  //radio item container style
        normal:PropTypes.shape({  //未选中状态
            icon:PropTypes.node,   //未选中的icon
            txtContainerStyle:viewStylePropType,
            txtStyle:textStylePropType,
        }),
        checked:PropTypes.shape({ //选中的状态
            icon:PropTypes.node,  //选中的icon
            txtContainerStyle:viewStylePropType,
            txtStyle:textStylePropType,
        }),
        normals:PropTypes.arrayOf(PropTypes.shape({ //多个未选中的状态
            icon:PropTypes.node,   //未选中的icon
            txtContainerStyle:viewStylePropType,
            txtStyle:textStylePropType,
        })),
        checkeds:PropTypes.arrayOf(PropTypes.shape({ //多个选中的状态
            icon:PropTypes.node,  //选中的icon
            txtContainerStyle:viewStylePropType,
            txtStyle:textStylePropType,
        })),
        iconStyle:imageStylePropTypes,
        iconResizeMode: Image.propTypes.resizeMode, //Image resizeMode
        space:PropTypes.number,   //间隔
        onPress:PropTypes.func,   //点击回调
        defaultIndex:PropTypes.number,    //默认选中项
        onItemLayout:PropTypes.func,    //item 的 onLayout 事件
        badge:PropTypes.oneOfType([
            PropTypes.element,
            PropTypes.func,
        ]),
    }

    static defaultProps = {
        data:[],
        style:{
            height: 50,
        },
        normal:{
            icon:require('./images/pop_btn_normal.png'),
            txtContainerStyle:{},
            txtStyle:{
                fontSize: 16,
                color:'black',
            }
        },
        checked:{
            icon:require('./images/pop_btn_selected.png'),
            txtContainerStyle:{},
            txtStyle:{
                fontSize: 16,
                color:'black',
            }
        },
        space:0,
        iconSize:{
            width:25,
            height:25,
        },
        defaultIndex:-1,
        iconResizeMode:'center',
    }

    constructor(props){
        super(props);
        this.state = {
            selectedRadioIndex:this.props.defaultIndex,
        }
    }

    /**
     * 选中click
     * @private
     */
    _radioCheckedClick = (e)=>{
        let radioIndex = this._getRadioIndexByEvent(e);

        if(radioIndex>=0){
            this.setState({
                selectedRadioIndex:radioIndex,
            }, ()=>{
                this._notificationPress(e);
            });
        }else{
            this._notificationPress(e);
        }
    }

    /**
     * 通知 press
     * @param radioIndex
     * @private
     */
    _notificationPress = (e)=>{
        this.props.onPress && this.props.onPress(e, this.checkedIndex, this.checkedData);
    }

    /**
     * 设置被选中项
     * @param index
     * @param isPerformPress    是否自动调用press
     */
    setCheck(index : number, isPerformPress : boolean = true){
        this.setState({
            selectedRadioIndex:index,
        }, ()=>{
            isPerformPress && this._notificationPress({});
        });
    }

    /**
     * 获取选中的数据
     */
    get checkedData(){
        return this.props.data[this.state.selectedRadioIndex];
    }

    /**
     * 获取选中的index
     * @returns {Number|*}
     */
    get checkedIndex(){
        return this.state.selectedRadioIndex;
    }

    /**
     * 通过事件对象获取 radioIndex
     * @private
     */
    _getRadioIndexByEvent = (e)=>{
        let radioIndex = -1;
        try {
            radioIndex = e._targetInst.memoizedProps.radioIndex;
        } catch (e) {
            console.error(e);
        }
        return radioIndex;
    }

    /**
     *
     * @param e
     * @private
     */
    _radioItemLayout = (e)=>{
        this.props.onItemLayout && this.props.onItemLayout(e, this._getRadioIndexByEvent(e));
    }

    render(){

        const {style, data, normal, checked, normals, checkeds, space, iconStyle, iconResizeMode, radioItemContainerStyle, badge, ...other} = this.props;

        if(!Array.isArray(data)){
            return ;
        }

        return (
            <View style={[styles.container, style]} {...other} pointerEvents={'box-none'}>
                {
                    data.map((item, index)=>{
                        let currStyle;
                        if(index === this.state.selectedRadioIndex){    //选中
                            currStyle = checkeds && checkeds[index];
                            if(!currStyle){
                                currStyle = checked;
                            }
                        }else{ //正常
                            currStyle = normals && normals[index];
                            if(!currStyle){
                                currStyle = normal;
                            }
                        }

                        let image = currStyle.icon ? (<Image resizeMode={iconResizeMode} fadeDuration={0} source={currStyle.icon} style={[styles.icon, iconStyle]}/>) : null;
                        let spaceView = (<View style={{width: index != data.length-1 ? space : 0, backgroundColor:'transparent'}}/>);
                        return (
                            <TouchableWithoutFeedback onLayout={this._radioItemLayout} onPress={this._radioCheckedClick} key={index}>
                                <View radioIndex={index} style={[styles.radioItem, radioItemContainerStyle]}>
                                    <View style={[styles.txtContainerStyle, currStyle.txtContainerStyle]}>
                                        {image}
                                        <WMText text={item.name} style={[currStyle.txtStyle]}/>
                                        {typeof badge === 'function' ? badge(item, index) : badge}
                                    </View>
                                    {spaceView}
                                </View>
                            </TouchableWithoutFeedback>
                        );
                    })
                }
            </View>
        );
    }

}

const styles = {
    container:{
        flexDirection:'row',
        height:50,
    },
    radioItem:{
        flexDirection:'row',
        justifyContent:'center',
        alignItems:'center',
        height:'100%',
    },
    txtContainerStyle:{
        flexDirection:'row',
        justifyContent:'center',
        alignItems:'center',
        height:'100%',
    },
    icon:{
        marginRight:10
    }
}

==> wm-react-native-components-0.0.81.tgz: package/WMRadioGroup.js <==
/**
 * Created with IntelliJ IDEA.
 * User: aaronlin
 * Date: 2017/11/22
 * Time: 13:37
 */
import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';

import {
    View,
    Text,
    Image,
    TouchableWithoutFeedback,
    ViewPropTypes,
} from 'react-native';

const cloneReferencedElement = require('react-clone-referenced-element');

import WMText from './WMText';

import {
    StyleSheetPropType,
    ViewStylePropTypes,
    TextStylePropTypes,
    ImageStylePropTypes,
} from './RNExt';

const viewStylePropType = StyleSheetPropType(ViewStylePropTypes);
const textStylePropType = StyleSheetPropType(TextStylePropTypes);
const imageStylePropTypes = StyleSheetPropType(ImageStylePropTypes);

/**
 * 单选Radio
 */
export default class WMRadioGroup extends PureComponent {

    static propTypes = {
        ...ViewPropTypes,
        data:PropTypes.arrayOf(PropTypes.shape({    //数据
            name:PropTypes.string.isRequired, //用来显示的name
        })),
        radioItemContainerStyle:viewStylePropType,  //radio item container style
        normal:PropTypes.shape({  //未选中状态
            icon:PropTypes.node,   //未选中的icon
            txtContainerStyle:viewStylePropType,
            txtStyle:textStylePropType,
        }),
        checked:PropTypes.shape({ //选中的状态
            icon:PropTypes.node,  //选中的icon
            txtContainerStyle:viewStylePropType,
            txtStyle:textStylePropType,
        }),
        normals:PropTypes.arrayOf(PropTypes.shape({ //多个未选中的状态
            icon:PropTypes.node,   //未选中的icon
            txtContainerStyle:viewStylePropType,
            txtStyle:textStylePropType,
        })),
        checkeds:PropTypes.arrayOf(PropTypes.shape({ //多个选中的状态
            icon:PropTypes.node,  //选中的icon
            txtContainerStyle:viewStylePropType,
            txtStyle:textStylePropType,
        })),
        iconStyle:imageStylePropTypes,
        iconResizeMode: Image.propTypes.resizeMode, //Image resizeMode
        space:PropTypes.number,   //间隔
        onPress:PropTypes.func,   //点击回调
        defaultIndex:PropTypes.number,    //默认选中项
        onItemLayout:PropTypes.func,    //item 的 onLayout 事件
        badge:PropTypes.oneOfType([
            PropTypes.element,
            PropTypes.func,
        ]),
    }

    static defaultProps = {
        data:[],
        style:{
            height: 50,
        },
        normal:{
            icon:require('./images/pop_btn_normal.png'),
            txtContainerStyle:{},
            txtStyle:{
                fontSize: 16,
                color:'black',
            }
        },
        checked:{
            icon:require('./images/pop_btn_selected.png'),
            txtContainerStyle:{},
            txtStyle:{
                fontSize: 16,
                color:'black',
            }
        },
        space:0,
        iconSize:{
            width:25,
            height:25,
        },
        defaultIndex:-1,
        iconResizeMode:'center',
    }

    /**
     * 缓存 radioItemTxt Layout信息
     * @type {{}}
     * @private
     */
    _cacheRadioItemTxtLayoutInfo = {};

    constructor(props){
        super(props);
        this.state = {
            selectedRadioIndex:this.props.defaultIndex,
        }
    }

    /**
     * 选中click
     * @private
     */
    _radioCheckedClick = (e)=>{
        let radioIndex = this._getRadioIndexByEvent(e);

        if(radioIndex>=0){
            this.setState({
                selectedRadioIndex:radioIndex,
            }, ()=>{
                this._notificationPress(e);
            });
        }else{
            this._notificationPress(e);
        }
    }

    /**
     * 通知 press
     * @param radioIndex
     * @private
     */
    _notificationPress = (e)=>{
        this.props.onPress && this.props.onPress(e, this.checkedIndex, this.checkedData);
    }

    /**
     * 设置被选中项
     * @param index
     * @param isPerformPress    是否自动调用press
     */
    setCheck(index : number, isPerformPress : boolean = true){
        this.setState({
            selectedRadioIndex:index,
        }, ()=>{
            isPerformPress && this._notificationPress({});
        });
    }

    /**
     * 获取选中的数据
     */
    get checkedData(){
        return this.props.data[this.state.selectedRadioIndex];
    }

    /**
     * 获取选中的index
     * @returns {Number|*}
     */
    get checkedIndex(){
        return this.state.selectedRadioIndex;
    }

    /**
     * 通过事件对象获取 radioIndex
     * @private
     */
    _getRadioIndexByEvent = (e)=>{
        let radioIndex = -1;
        try {
            radioIndex = e._targetInst.memoizedProps.radioIndex;
        } catch (e) {
            console.error(e);
        }
        return radioIndex;
    }

    /**
     *
     * @param e
     * @private
     */
    _radioItemLayout = (e)=>{
        this.props.onItemLayout && this.props.onItemLayout(e, this._getRadioIndexByEvent(e));
    }

    /**
     *
     * @param e
     * @private
     */
    _radioItemTxtLayout = (e)=>{
        const radioIndex = this._getRadioIndexByEvent(e);
        if(radioIndex >= 0){
            this._cacheRadioItemTxtLayoutInfo['radioItem_'+radioIndex] = e.nativeEvent.layout;
        }
    }

    /**
     *
     * @param e
     * @private
     */
    _badgeLayout = (e)=>{
        const radioIndex = this._getRadioIndexByEvent(e);
        if(radioIndex >= 0){
            let radioItemBadge = this.refs['radioItemBadge_'+radioIndex];
            if(radioItemBadge){
                const { style } = radioItemBadge.props;
                let styleLeft = (style && style.left && !Number.isNaN(Number(style.left))) ? style.left : 0;
                let layout = this._cacheRadioItemTxtLayoutInfo['radioItem_'+radioIndex];
                let width = layout ? layout.width : 0;
                radioItemBadge.setNativeProps({
                    style:{
                        left: styleLeft + width,
                    }
                });
            }
        }
    }

    render(){

        const {style, data, normal, checked, normals, checkeds, space, iconStyle, iconResizeMode, radioItemContainerStyle, badge, ...other} = this.props;

        if(!Array.isArray(data)){
            return ;
        }

        return (
            <View style={[styles.container, style]} {...other} pointerEvents={'box-none'}>
                {
                    data.map((item, index)=>{
                        let currStyle;
                        if(index === this.state.selectedRadioIndex){    //选中
                            currStyle = checkeds && checkeds[index];
                            if(!currStyle){
                                currStyle = checked;
                            }
                        }else{ //正常
                            currStyle = normals && normals[index];
                            if(!currStyle){
                                currStyle = normal;
                            }
                        }

                        let radioItemBadge = typeof badge === 'function' ? badge(item, index) : badge;
                        if(radioItemBadge){
                            radioItemBadge = cloneReferencedElement(radioItemBadge, {ref:'radioItemBadge_'+index, radioIndex:index, onLayout:this._badgeLayout});
                        }

                        let image = currStyle.icon ? (<Image resizeMode={iconResizeMode} fadeDuration={0} source={currStyle.icon} style={[styles.icon, iconStyle]}/>) : null;
                        let spaceView = (<View style={{width: index != data.length-1 ? space : 0, backgroundColor:'transparent'}}/>);
                        return (
                            <TouchableWithoutFeedback onLayout={this._radioItemLayout} onPress={this._radioCheckedClick} key={index}>
                                <View radioIndex={index} style={[styles.radioItem, radioItemContainerStyle]}>
                                    <View radioIndex={index} onLayout={this._radioItemTxtLayout} style={[styles.txtContainerStyle, currStyle.txtContainerStyle]}>
                                        {image}
                                        <WMText text={item.name} style={[currStyle.txtStyle]}/>
                                    </View>

                                    {radioItemBadge}

                                    {spaceView}
                                </View>
                            </TouchableWithoutFeedback>
                        );
                    })
                }
            </View>
        );
    }

}

const styles = {
    container:{
        flexDirection:'row',
        height:50,
    },
    radioItem:{
        flexDirection:'row',
        justifyContent:'center',
        alignItems:'center',
        height:'100%',
    },
    txtContainerStyle:{
        flexDirection:'row',
        justifyContent:'center',
        alignItems:'center',
        height:'100%',
    },
    icon:{
        marginRight:10
    }
}
